"""
txtulip: Twisted reactor based on Tulip/Asyncio/Trollius
========================================================

``txtulip`` is licensed under the MIT open source license, and maintained by
Itamar Turner-Trauring.

``txtulip`` can be found at https://github.com/itamarst/txtulip
"""
